;;; ebuild-mode.el --- a mode for editing Gentoo GNU/Linux .ebuild and .eclass files.

;; Copyright (C) 2003  Free Software Foundation, Inc.

;; Author:  <mkennedy@gentoo.org>
;; Keywords: convenience

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; The commands have been grouped into lists of rough similarity.  If
;; you can think of a better way to arrange these, please let us know.
;; We map each set of keywords to the basic faces: font-lock-*-face.

;;; Code:


(defvar ebuild-commands-0
  '("dobin" "dodir" "dodoc" "doexe" "dohard" "dohtml" "doinfo" "doins" "dojar"
    "dolib" "dolib.a" "dolib.so" "doman" "domo" "donewins" "dopython" "dosbin"
    "dosed" "dosym" "find-packages" "find-requires" "fowners" "fperms" "newbin"
    "newdoc" "newexe" "newins" "newlib.a" "newman" "newsbin")
  "*List of commands from /usr/lib/portage/bin")

(defvar ebuild-commands-1
  '("filter-flags" "append-flags" "replace-flags" "is-flag" "strip-flags"
    "get-flag" "epatch" "enewuser" "enewgroup" "edos2unix")
  "*List of commands from /usr/lib/portage/bin/extra_functions.sh")

(defvar ebuild-commands-2
  '("esyslog" "ewarn" "einfo" "use" "has" "has_version" "best_version" "use_with"
    "use_enable" "check_KV" "keepdir")
  "*List of commands from /usr/lib/portage/bin/ebuild.sh")

(defvar ebuild-commands-3
  '("addread" "addwrite" "adddeny" "addpredict")
  "*List of commands from /usr/lib/portage/bin/ebuild.sh")

(defvar ebuild-commands-4
  '("unpack" "econf" "einstall" "emake")
  "*List of commands from /usr/lib/portage/bin/ebuild.sh")

(defvar ebuild-commands-5
  '("into" "insinto" "exeinto" "docinto" "insopts" "diropts" "exeopts" "libopts")
  "*List of commands from /usr/lib/portage/bin/ebuild.sh")

(defvar ebuild-commands-6
  '("inherit" "newdepend" "newrdepend" "newcdepend" "newpdepend")
  "*List of commands from /usr/lib/portage/bin/ebuild.sh")

(defun ebuild-make-keywords-list (keywords-list face &optional prefix suffix)
  ;; based on `make-generic-keywords-list' from  generic.el
  (unless (listp keywords-list)
    (error "Keywords argument must be a list of strings"))
  (cons (concat prefix "\\<"
		(regexp-opt keywords-list t)
		"\\>" suffix)
	face))

(define-derived-mode ebuild-mode shell-script-mode "Ebuild"
  "Major mode for Gentoo GNU/Linux .ebuild and .eclass files"
  (font-lock-add-keywords 'ebuild-mode
   (list (ebuild-make-keywords-list ebuild-commands-0 'font-lock-type-face)
         (ebuild-make-keywords-list ebuild-commands-1 'font-lock-type-face)
         (ebuild-make-keywords-list ebuild-commands-2 'font-lock-type-face)
         (ebuild-make-keywords-list ebuild-commands-3 'font-lock-warning-face)
         (ebuild-make-keywords-list ebuild-commands-4 'font-lock-type-face)
         (ebuild-make-keywords-list ebuild-commands-5 'font-lock-builtin-face)
         (ebuild-make-keywords-list ebuild-commands-6 'font-lock-type-face)))
  (add-hook 'write-file-functions 'delete-trailing-whitespace)
  (setq tab-width 4
        indent-tabs-mode t))

(add-to-list 'auto-mode-alist '("\\.ebuild\\'" . ebuild-mode))
(add-to-list 'auto-mode-alist '("\\.eclass\\'" . ebuild-mode))

(define-derived-mode gentoo-conf-mode shell-script-mode "Gentoo"
  "Major mode for editing Gentoo GNU/Linux configuration files such as make.conf.")

(add-to-list 'auto-mode-alist '("\\`\\(/etc/make.conf\\|/etc/make.globals\\)\\'" . gentoo-conf-mode))

(provide 'gentoo-ebuild)


;;; Local Variables: ***
;;; indent-tabs-mode: nil ***
;;; End: ***

;;; ebuild-mode.el ends here
